/*
 * FILE:
 *   com_ptr_wrapper.hpp
 *
 * PURPOSE:
 *   Class that automatically Release()s a COM pointer in the destructor.
 *   (Similar to CComPtr but avoids the dependency on ATL.)
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * CREATED:
 *   20-Oct-2011
 *   
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2008 - 2011.
 *
 *   This file is part of the "Coercri" software library. Usage of "Coercri"
 *   is permitted under the terms of the Boost Software License, Version 1.0, 
 *   the text of which is displayed below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 */

#ifndef COERCRI_COM_PTR_WRAPPER_HPP
#define COERCRI_COM_PTR_WRAPPER_HPP

#include "boost/noncopyable.hpp"

namespace Coercri {

    template<class T>
    class ComPtrWrapper : boost::noncopyable {
    public:
        ComPtrWrapper() : p(0) { }
        explicit ComPtrWrapper(T* p_) : p(p_) { }
        ~ComPtrWrapper() { if (p) p->Release(); }
        
        void reset(T* p_) { if (p) p->Release(); p = p_; }
        
        T& operator*() { return *p; }
        const T& operator*() const { return *p; }
        T* operator->() { return p; }
        const T* operator->() const { return p; }
        T* get() { return p; }
        const T* get() const { return p; }
        
    private:
        T* p;
    };
    
}

#endif
