/*
 * FILE:
 *   sdl_ttf_font.hpp
 *
 * PURPOSE:
 *   Implementation of Font using SDL_TTF
 *
 * AUTHOR:
 *   Stephen Thompson <stephen@solarflare.org.uk>
 *
 * COPYRIGHT:
 *   Copyright (C) Stephen Thompson, 2008 - 2009.
 *
 *   This file is part of the "Coercri" software library. Usage of "Coercri"
 *   is permitted under the terms of the Boost Software License, Version 1.0, 
 *   the text of which is displayed below.
 *
 *   Boost Software License - Version 1.0 - August 17th, 2003
 *
 *   Permission is hereby granted, free of charge, to any person or organization
 *   obtaining a copy of the software and accompanying documentation covered by
 *   this license (the "Software") to use, reproduce, display, distribute,
 *   execute, and transmit the Software, and to prepare derivative works of the
 *   Software, and to permit third-parties to whom the Software is furnished to
 *   do so, all subject to the following:
 *
 *   The copyright notices in the Software and this entire statement, including
 *   the above license grant, this restriction and the following disclaimer,
 *   must be included in all copies of the Software, in whole or in part, and
 *   all derivative works of the Software, unless such copies or derivative
 *   works are solely in the form of machine-executable object code generated by
 *   a source language processor.
 *
 *   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 *   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 *   FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 *   SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 *   FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 *   DEALINGS IN THE SOFTWARE.
 *
 * NOTES:
 *   Please note that this file will only work in conjunction with the SDL_TTF
 *   library, which is licensed under the LGPL.
 *
 */

#ifndef COERCRI_SDL_TTF_FONT_HPP
#define COERCRI_SDL_TTF_FONT_HPP

#include "../../gfx/font.hpp"
#include "../../gfx/color.hpp"

#include "SDL_ttf.h"
#include "boost/shared_ptr.hpp"
#include <iosfwd>

namespace Coercri {

    class SDLTTFFont : public Font {
    public:
        // "size" is the point size based on 72DPI -- which basically translates to pixel height.
        // NOTE: SDL_TTF may keep the file open and lazily load data
        // as needed. This is why the ifstream is passed in a
        // shared_ptr.
        SDLTTFFont(boost::shared_ptr<std::istream> str, int size);

        virtual void getTextSize(const std::string &text, int &w, int &h) const;
        virtual int getTextHeight() const;

        void drawText(GfxContext &dest, int x, int y, const std::string &text, Color col) const;
        
    private:
        boost::shared_ptr<TTF_Font> font;  // constructed with DeleteTTFFont
    };

}

#endif
